import { shallowMount } from '@vue/test-utils';
import { popupManager } from '@inkline/inkline/src/factories/PopupManager';
import { IModal } from '@inkline/inkline/src/components';

describe('Components', () => {
    describe('IModal', () => {
        let wrapper;

        beforeEach(() => {
            wrapper = shallowMount(IModal, {
                propsData: {
                    id: 'modal'
                },
                slots: {
                    default: ['<div/>']
                },
                methods: {
                    created: IModal.created,
                    destroyed: IModal.destroyed
                }
            });
        });

        it('should render correctly', () => {
            expect(wrapper.html()).toMatchSnapshot();
        });

        describe('props', () => {
            describe('transition', () => {
                it('should be defined', () => {
                    expect(wrapper.vm.transition).toBeDefined();
                    expect(wrapper.vm.transition).toEqual('zoom-in-center-transition');
                });
            });

            describe('showClose', () => {
                it('should be defined', () => {
                    expect(wrapper.vm.showClose).toBeDefined();
                    expect(wrapper.vm.showClose).toEqual(true);
                });
            });

            describe('closeOnPressEscape', () => {
                it('should be defined', () => {
                    expect(wrapper.vm.closeOnPressEscape).toBeDefined();
                    expect(wrapper.vm.closeOnPressEscape).toEqual(true);
                });
            });

            describe('fill', () => {
                it('should be defined', () => {
                    expect(wrapper.vm.fill).toBeDefined();
                    expect(wrapper.vm.fill).toEqual(false);
                });
            });

            describe('value', () => {
                it('should be defined', () => {
                    expect(wrapper.vm.value).toBeDefined();
                    expect(wrapper.vm.value).toEqual(false);
                });
            });
        });

        describe('computed', () => {
            describe('visible', () => {
                it('should be defined', () => {
                    expect(wrapper.vm.visible).toBeDefined();
                });

                it('should equal value', () => {
                    expect(wrapper.vm.visible).toEqual(wrapper.vm.value);

                    wrapper.setProps({ value: true });

                    expect(wrapper.vm.visible).toEqual(wrapper.vm.value);
                });
            });
        });

        describe('data', () => {
            describe('basename', () => {
                it('should be defined', () => {
                    expect(wrapper.vm.basename).toBeDefined();
                    expect(wrapper.vm.basename).toEqual('modal');
                });
            });

            describe('id', () => {
                it('should be defined', () => {
                    expect(wrapper.vm.id).toBeDefined();
                    expect(wrapper.vm.id).toEqual('modal');
                });

                it('should be randomly generated if not defined', () => {
                    wrapper = shallowMount(IModal, {
                        slots: {
                            default: ['<div/>']
                        }
                    });

                    expect(wrapper.vm.id).toMatch(/^modal-\w+/);
                });
            });
        });

        describe('methods', () => {
            describe('show()', () => {
                it('should be defined', () => {
                    expect(wrapper.vm.show).toBeDefined();
                });

                it('should set value to true', () => {
                    const spy = jest.spyOn(wrapper.vm, '$emit');

                    wrapper.vm.show();

                    expect(spy).toHaveBeenCalled();
                    expect(spy).toHaveBeenCalledWith('input', true);
                });

                it('should not set value to true if disabled', () => {
                    const spy = jest.spyOn(wrapper.vm, '$emit');

                    wrapper.setProps({ disabled: true });
                    wrapper.vm.show();

                    expect(spy).not.toHaveBeenCalled();
                });

                it('should emit show event', () => {
                    const spy = jest.spyOn(wrapper.vm, '$emit');

                    wrapper.vm.show();

                    expect(spy).toHaveBeenCalled();
                    expect(spy).toHaveBeenCalledWith('show', wrapper.vm);
                });

                it('should call popupManager openModal', () => {
                    const spy = jest.spyOn(popupManager, 'openModal');

                    wrapper.vm.show();

                    expect(spy).toHaveBeenCalled();
                    expect(spy).toHaveBeenCalledWith(wrapper.vm.id);
                });
            });

            describe('hide()', () => {
                it('should be defined', () => {
                    expect(wrapper.vm.hide).toBeDefined();
                });

                it('should set value to false', () => {
                    const spy = jest.spyOn(wrapper.vm, '$emit');

                    wrapper.setProps({ value: true });
                    wrapper.vm.hide();

                    expect(spy).toHaveBeenCalled();
                    expect(spy).toHaveBeenCalledWith('input', false);
                });

                it('should not set value to false if disabled', () => {
                    const spy = jest.spyOn(wrapper.vm, '$emit');

                    wrapper.setProps({ disabled: true });
                    wrapper.setProps({ value: true });
                    wrapper.vm.hide();

                    expect(spy).not.toHaveBeenCalled();
                });

                it('should emit hide event', () => {
                    const spy = jest.spyOn(wrapper.vm, '$emit');

                    wrapper.setProps({ value: true });
                    wrapper.vm.hide();

                    expect(spy).toHaveBeenCalled();
                    expect(spy).toHaveBeenLastCalledWith('hide', wrapper.vm);
                });

                it('should call popupManager closeModal', () => {
                    const spy = jest.spyOn(popupManager, 'closeModal');

                    wrapper.setProps({ value: true });
                    wrapper.vm.hide();

                    expect(spy).toHaveBeenCalled();
                    expect(spy).toHaveBeenCalledWith(wrapper.vm.id);
                });
            });
        });

        describe('watch', () => {
            describe('value()', () => {
                it('should not call show() or hide() if disabled', (done) => {
                    const spy1 = jest.spyOn(wrapper.vm, 'show');
                    const spy2 = jest.spyOn(wrapper.vm, 'hide');

                    wrapper.setProps({ disabled: true });

                    wrapper.vm.$nextTick(() => {
                        wrapper.setProps({ value: true });

                        wrapper.vm.$nextTick(() => {
                            wrapper.setProps({ value: false });

                            wrapper.vm.$nextTick(() => {
                                expect(spy1).not.toHaveBeenCalled();
                                expect(spy2).not.toHaveBeenCalled();
                                done();
                            });
                        });
                    });
                });

                it('should call show() if value changes to true', (done) => {
                    const spy = jest.spyOn(wrapper.vm, 'show');

                    wrapper.setProps({ value: true });

                    wrapper.vm.$nextTick(() => {
                        expect(spy).toHaveBeenCalled();
                        done();
                    });
                });

                it('should call hide() if value changes to false', (done) => {
                    const spy = jest.spyOn(wrapper.vm, 'hide');

                    wrapper.setProps({ value: true });
                    wrapper.vm.$nextTick(() => {
                        wrapper.setProps({ value: false });
                        wrapper.vm.$nextTick(() => {
                            expect(spy).toHaveBeenCalled();
                            done();
                        });
                    });
                });
            });
        });

        describe('created()', () => {
            it('should be defined', () => {
                expect(IModal.created).toBeDefined();
            });

            it('should register the modal to the popup manager', () => {
                const spy = jest.spyOn(popupManager, 'register');

                wrapper.vm.created();

                expect(spy).toHaveBeenCalled();
                expect(spy).toHaveBeenCalledWith(wrapper.vm);
            });


            it('should add class rules to classes provider', () => {
                const classRulesLength = wrapper.vm.classesProvider.length;

                wrapper.vm.created();
                expect(wrapper.vm.classesProvider.length).toEqual(classRulesLength + 1)
            });

            it('should add "-fill" class if "fill" property is true', () => {
                const rule = wrapper.vm.classesProvider[wrapper.vm.classesProvider.length - 1];

                expect(rule()).toEqual(expect.objectContaining({ '-fill': false }));
                wrapper.setProps({ fill: true });
                expect(rule()).toEqual(expect.objectContaining({ '-fill': true }));
            });

        });

        describe('destroyed()', () => {
            it('should be defined', () => {
                expect(IModal.destroyed).toBeDefined();
            });

            it('should unregister the modal from the popup manager', () => {
                const spy = jest.spyOn(popupManager, 'unregister');

                wrapper.vm.destroyed();

                expect(spy).toHaveBeenCalled();
                expect(spy).toHaveBeenCalledWith(wrapper.vm);
            });
        });
    });
});
